"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createReadModelsReducer;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _objectSpread8 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _assign = _interopRequireDefault(require("@babel/runtime/core-js/object/assign"));

var _getPrototypeOf = _interopRequireDefault(require("@babel/runtime/core-js/object/get-prototype-of"));

var _create = _interopRequireDefault(require("@babel/runtime/core-js/object/create"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _getIterator2 = _interopRequireDefault(require("@babel/runtime/core-js/get-iterator"));

var _defineProperty3 = _interopRequireDefault(require("@babel/runtime/core-js/object/define-property"));

var _getOwnPropertyDescriptor = _interopRequireDefault(require("@babel/runtime/core-js/object/get-own-property-descriptor"));

var _getOwnPropertyNames = _interopRequireDefault(require("@babel/runtime/core-js/object/get-own-property-names"));

var _action_types = require("./action_types");

var _diffJson = require("diff-json");

function copyResolveSerials(oldState, newState) {
  (0, _getOwnPropertyNames.default)(oldState).forEach(function (key) {
    var propertyDescriptor = (0, _getOwnPropertyDescriptor.default)(oldState, key);

    if (key.indexOf('resolve-serial:') !== 0 || propertyDescriptor.enumerable || propertyDescriptor.writable) {
      return;
    }

    (0, _defineProperty3.default)(newState, key, propertyDescriptor);
  });
}

function refreshUpdatedObjects(updatedObject, changes) {
  var embeddedKey = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '$index';
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    var _loop = function _loop() {
      var _ref = _step.value;
      var key = _ref.key,
          nextChanges = _ref.changes,
          nextEmbeddedKey = _ref.embeddedKey;
      var calcKey = embeddedKey !== '$index' && Array.isArray(updatedObject) ? updatedObject.reduce(function (result, value, idx) {
        return value[embeddedKey] === key ? idx : result;
      }, 0) : key;

      if (updatedObject[calcKey] == null || updatedObject[calcKey].constructor === String || updatedObject[calcKey].constructor === Number || updatedObject[calcKey].constructor === Date) {
        return "continue";
      }

      if (Array.isArray(updatedObject[calcKey])) {
        updatedObject[calcKey] = (0, _toConsumableArray2.default)(updatedObject[calcKey]);
      } else {
        var nextObject = (0, _create.default)((0, _getPrototypeOf.default)(updatedObject[calcKey]));
        (0, _assign.default)(nextObject, updatedObject[calcKey]);
        updatedObject[calcKey] = nextObject;
      }

      if (Array.isArray(nextChanges)) {
        refreshUpdatedObjects(updatedObject[calcKey], nextChanges, nextEmbeddedKey);
      }
    };

    for (var _iterator = (0, _getIterator2.default)(changes), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _ret = _loop();

      if (_ret === "continue") continue;
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
}

function createReadModelsReducer() {
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var action = arguments.length > 1 ? arguments[1] : undefined;

    switch (action.type) {
      case _action_types.READMODEL_LOAD_INITIAL_STATE:
        {
          var readModelName = action.readModelName,
              resolverName = action.resolverName,
              initialState = action.initialState,
              serialId = action.serialId;
          var nextState = (0, _objectSpread8.default)({}, state, (0, _defineProperty2.default)({}, readModelName, (0, _objectSpread8.default)({}, state[readModelName] || {}, (0, _defineProperty2.default)({}, resolverName, initialState))));
          copyResolveSerials(state, nextState);
          Object.defineProperty(nextState, "resolve-serial:".concat(readModelName, ":").concat(resolverName), {
            configurable: true,
            writable: false,
            enumerable: false,
            value: serialId
          });
          return nextState;
        }

      case _action_types.READMODEL_DROP_STATE:
        {
          var _readModelName = action.readModelName,
              _resolverName = action.resolverName;

          var _nextState = (0, _objectSpread8.default)({}, state, (0, _defineProperty2.default)({}, _readModelName, (0, _objectSpread8.default)({}, state[_readModelName] || {}, (0, _defineProperty2.default)({}, _resolverName, null))));

          copyResolveSerials(state, _nextState);
          delete _nextState["resolve-serial:".concat(_readModelName, ":").concat(_resolverName)];
          delete _nextState[_readModelName][_resolverName];
          return _nextState;
        }

      case _action_types.READMODEL_SUBSCRIPTION_DIFF:
        {
          var _readModelName2 = action.readModelName,
              _resolverName2 = action.resolverName,
              _serialId = action.serialId,
              diff = action.diff;

          if (!state[_readModelName2] || !state[_readModelName2].hasOwnProperty(_resolverName2) || state["resolve-serial:".concat(_readModelName2, ":").concat(_resolverName2)] !== _serialId) {
            return state;
          }

          var wrappedResolverState = {
            wrap: state[_readModelName2][_resolverName2]
          };
          (0, _diffJson.applyChanges)(wrappedResolverState, diff);
          refreshUpdatedObjects(wrappedResolverState, diff);

          var _nextState2 = (0, _objectSpread8.default)({}, state, (0, _defineProperty2.default)({}, _readModelName2, (0, _objectSpread8.default)({}, state[_readModelName2], (0, _defineProperty2.default)({}, _resolverName2, wrappedResolverState.wrap))));

          copyResolveSerials(state, _nextState2);
          return _nextState2;
        }

      default:
        return state;
    }
  };
}